'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.roots = exports.render = void 0;
const core_1 = require('@dark-engine/core');
const dom_1 = require('../dom');
const portal_1 = require('../portal');
const scheduler_1 = require('../scheduler');
let isInjected = false;
const roots = new Map();
exports.roots = roots;
function inject() {
  core_1.platform.createNativeElement = dom_1.createNativeElement;
  core_1.platform.requestAnimationFrame = requestAnimationFrame.bind(this);
  core_1.platform.cancelAnimationFrame = cancelAnimationFrame.bind(this);
  core_1.platform.scheduleCallback = scheduler_1.scheduleCallback;
  core_1.platform.shouldYeildToHost = scheduler_1.shouldYeildToHost;
  core_1.platform.applyCommit = dom_1.applyCommit;
  core_1.platform.finishCommitWork = dom_1.finishCommitWork;
  core_1.platform.detectIsDynamic = () => true;
  core_1.platform.detectIsPortal = portal_1.detectIsPortal;
  core_1.platform.unmountPortal = portal_1.unmountPortal;
  core_1.platform.restart = () => {};
  isInjected = true;
}
function render(element, container, hydrate = false) {
  !isInjected && inject();
  if (!(container instanceof Element)) {
    throw new Error(`[Dark]: render receives only Element as container!`);
  }
  const isMounted = !(0, core_1.detectIsUndefined)(roots.get(container));
  let rootId = null;
  if (!isMounted) {
    rootId = roots.size;
    roots.set(container, rootId);
    if (!hydrate) {
      container.innerHTML = '';
    }
  } else {
    rootId = roots.get(container);
  }
  if (core_1.isInsertionEffectsZone.get(rootId)) return;
  const callback = () => {
    core_1.rootStore.set(rootId);
    const currentRoot = core_1.currentRootStore.get();
    const isUpdate = Boolean(currentRoot);
    const fiber = new core_1.Fiber().mutate({
      nativeElement: container,
      instance: new core_1.TagVirtualNode(
        core_1.ROOT,
        {},
        (0, core_1.flatten)([element || (0, core_1.createReplacer)()]),
      ),
      alternate: currentRoot,
      effectTag: isUpdate ? core_1.EffectTag.UPDATE : core_1.EffectTag.CREATE,
    });
    core_1.fiberMountStore.reset();
    core_1.wipRootStore.set(fiber);
    core_1.isHydrateZone.set(hydrate);
    core_1.nextUnitOfWorkStore.set(fiber);
  };
  core_1.platform.scheduleCallback(callback, {
    priority: core_1.TaskPriority.NORMAL,
    forceSync: hydrate || core_1.isLayoutEffectsZone.get(),
  });
}
exports.render = render;
//# sourceMappingURL=render.js.map
