'use strict';
Object.defineProperty(exports, '__esModule', { value: true });
exports.scheduleCallback = exports.shouldYeildToHost = void 0;
const core_1 = require('@dark-engine/core');
const queueByPriority = {
  animations: [],
  hight: [],
  normal: [],
  low1: [],
  low2: [],
};
const YEILD_INTERVAL = 4;
const MAX_LOW_PRIORITY_TASKS_LIMIT = 100000;
let scheduledCallback = null;
let deadline = 0;
let isMessageLoopRunning = false;
let currentTask = null;
class Task {
  static nextTaskId = 0;
  id;
  time;
  timeoutMs;
  priority;
  forceSync;
  callback;
  constructor(options) {
    this.id = ++Task.nextTaskId;
    this.time = options.time;
    this.timeoutMs = options.timeoutMs;
    this.priority = options.priority;
    this.forceSync = options.forceSync;
    this.callback = options.callback;
  }
}
const shouldYeildToHost = () => (0, core_1.getTime)() >= deadline;
exports.shouldYeildToHost = shouldYeildToHost;
function scheduleCallback(callback, options) {
  const { priority = core_1.TaskPriority.NORMAL, timeoutMs = 0, forceSync = false } = options || {};
  const task = new Task({ time: (0, core_1.getTime)(), timeoutMs, priority, forceSync, callback });
  const map = {
    [core_1.TaskPriority.ANIMATION]: () => queueByPriority.animations.push(task),
    [core_1.TaskPriority.HIGH]: () => queueByPriority.hight.push(task),
    [core_1.TaskPriority.NORMAL]: () => queueByPriority.normal.push(task),
    [core_1.TaskPriority.LOW]: () =>
      task.timeoutMs > 0 ? queueByPriority.low2.push(task) : queueByPriority.low1.push(task),
  };
  map[task.priority]();
  executeTasks();
}
exports.scheduleCallback = scheduleCallback;
function pick(queue) {
  if (!queue.length) return false;
  currentTask = queue.shift();
  const isAnimation = currentTask.priority === core_1.TaskPriority.ANIMATION;
  currentTask.callback();
  if (currentTask.forceSync || isAnimation) {
    requestCallbackSync(core_1.workLoop);
  } else {
    requestCallback(core_1.workLoop);
  }
  return true;
}
function executeTasks() {
  const isBusy = (0, core_1.detectIsBusy)();
  if (!isBusy) {
    checkOverdueTasks() ||
      gc() ||
      pick(queueByPriority.animations) ||
      pick(queueByPriority.hight) ||
      pick(queueByPriority.normal) ||
      requestIdleCallback(() => pick(queueByPriority.low1) || pick(queueByPriority.low2));
  }
}
function gc() {
  if (queueByPriority.low1.length > MAX_LOW_PRIORITY_TASKS_LIMIT) {
    queueByPriority.low1 = [];
  }
  return false;
}
function checkOverdueTasks() {
  const [task] = queueByPriority.low2;
  if (task && (0, core_1.getTime)() - task.time > task.timeoutMs) {
    pick(queueByPriority.low2);
    return true;
  }
  return false;
}
function performWorkUntilDeadline() {
  if (scheduledCallback) {
    deadline = (0, core_1.getTime)() + YEILD_INTERVAL;
    try {
      const hasMoreWork = scheduledCallback();
      if (!hasMoreWork) {
        currentTask = null;
        isMessageLoopRunning = false;
        scheduledCallback = null;
        executeTasks();
      } else {
        port.postMessage(null);
      }
    } catch (error) {
      port.postMessage(null);
      throw error;
    }
  } else {
    isMessageLoopRunning = false;
  }
}
function requestCallback(callback) {
  if (process.env.NODE_ENV === 'test') {
    return requestCallbackSync(callback);
  }
  scheduledCallback = callback;
  if (!isMessageLoopRunning) {
    isMessageLoopRunning = true;
    port.postMessage(null);
  }
}
function requestCallbackSync(callback) {
  while (callback()) {}
  executeTasks();
  currentTask = null;
}
let channel = null;
let port = null;
function setup() {
  if (process.env.NODE_ENV === 'test') {
    return;
  }
  channel = new MessageChannel();
  port = channel.port2;
  channel.port1.onmessage = performWorkUntilDeadline;
}
setup();
//# sourceMappingURL=scheduler.js.map
