import { detectIsFunction, eventsStore } from '@dark-engine/core';
class SyntheticEvent {
  type = '';
  sourceEvent = null;
  target = null;
  propagation = true;
  constructor(options) {
    this.type = options.sourceEvent.type;
    this.sourceEvent = options.sourceEvent;
    this.target = options.target;
  }
  stopPropagation() {
    this.propagation = false;
    this.sourceEvent.stopPropagation();
  }
  preventDefault() {
    this.sourceEvent.preventDefault();
  }
  getPropagation() {
    return this.propagation;
  }
}
function delegateEvent(options) {
  const { target, eventName, handler } = options;
  const eventsMap = eventsStore.get();
  const handlerMap = eventsMap.get(eventName);
  if (!handlerMap) {
    const rootHandler = event => {
      const fireEvent = eventsMap.get(eventName).get(event.target);
      const target = event.target;
      let syntheticEvent = null;
      if (detectIsFunction(fireEvent)) {
        syntheticEvent = new SyntheticEvent({
          sourceEvent: event,
          target,
        });
        fireEvent(syntheticEvent);
      }
      if (syntheticEvent ? syntheticEvent.getPropagation() : target.parentElement) {
        target.parentElement.dispatchEvent(new event.constructor(event.type, event));
      }
    };
    eventsMap.set(eventName, new WeakMap([[target, handler]]));
    document.addEventListener(eventName, rootHandler, true);
    eventsStore.addUnsubscriber(() => document.removeEventListener(eventName, rootHandler, true));
  } else {
    handlerMap.set(target, handler);
  }
}
const detectIsEvent = attrName => attrName.startsWith('on');
const getEventName = attrName => attrName.slice(2, attrName.length).toLowerCase();
export { SyntheticEvent, delegateEvent, detectIsEvent, getEventName };
//# sourceMappingURL=events.js.map
